#include <vector>
#include "scene.h"


using namespace VgMath;

static char toChar(Scalar brightness){
    if (brightness<0.1) return ' ';
    if (brightness<0.3) return '.';
    if (brightness<0.5) return '-';
    if (brightness<0.7) return '+';
    if (brightness<0.9) return '*';
    return '#';
}


void Scene::raytrace1stPerson( int objIndex ) const{
    raytrace(objIndex , 40, 2 );
}

void Scene::raytraceExternalView() const{
    raytrace(-1 , 40, 2 );
}

// if objIndex is negative, use world view
void Scene::raytrace(int objIndex, int res, Scalar focal) const{

    Versor3 lightDir = Vector3( 2, 5, -5 ).normalized();

    if (objIndex>=0) lightDir = object[ objIndex ].transform.inverse()(lightDir);

    const Point3 pov (0,0,0); // because we are in VIEW space!


    SphereVector sss = toLocalOf( objIndex );


    std::string buffer;

    for (int yPixel=0; yPixel<res; yPixel++ ){
        for (int xPixel=0; xPixel<res; xPixel++){
            Scalar clipX = +Scalar(2)*xPixel/res - 1;
            Scalar clipY = -Scalar(2)*yPixel/res + 1;

            Point3 pixelPos(clipX, clipY, focal );
            Ray r( pov, pixelPos );

            char pixel = '.'; // background

            Versor3 n = Versor3::up();

            if (intersect(r,sss,n)) {
                Scalar diffuse = dot( n, lightDir );
                pixel = toChar( diffuse );
            }
            buffer.push_back( pixel );
            buffer.push_back( ' ' );
        }
        buffer.push_back( '\n' );
    }
    std::cout << buffer;
}



