#pragma once

#include <vector>

#include "shapes.h"
#include "transform.h"

namespace VgMath{

typedef std::vector<Sphere> SphereVector;

class GameObject{
public:
    Transform transform;

    // here: the content of this object, for now:
    // automatically a unit sphere centered in the origin
    // PLUS a secondary sphere with radius 0.6, centered in (0,0,0.3)
};

class Scene{
    void raytrace( int objectIndex, int res, Scalar focal  ) const;

public:
    std::vector<GameObject> object;

    void raytrace1stPerson( int objIndex ) const;
    void raytraceExternalView() const;

    void createRandom(int howManySpheres){
        object.clear();

        for (int i=0; i<howManySpheres; i++) {
            GameObject obj;

            obj.transform.translate = Point3::random(15).asVector3() + Vector3(0,0,25);
            obj.transform.translate.y = 0;
            obj.transform.scale = randomBetween( 1, 3 );
            obj.transform.rotate = Quaternion::angleAxis( randomBetween(0,360) , Versor3::up() );

            object.push_back( obj );
        }
    }

    // populates a vectors of spheres to be rendered, each in the local space of i
    // OR, in global space if i < 0
    SphereVector toLocalOf( int i ) const{

        // trasnform from world to local space of object i
        Transform toLocal;

        if (i<0) toLocal = Transform();
        else toLocal = object[i].transform.inverse() ;


        // for now, every game object contains...
        const Sphere head (Point3(0,0,0) , 1 );
        const Sphere nose (Point3(0,0,0.85) , 0.6 );
        // ... expressed in LOCAL space of each object

        SphereVector res;
        for ( const GameObject & obj : object) {
            res.push_back(  toLocal( obj.transform( head ) ) );
            res.push_back(  toLocal( obj.transform( nose ) ) );
        }
        return res;
    }



};


inline bool intersect(const Ray& r, const SphereVector& sphereVector, Versor3 &n){
    Scalar maxDist = 10000.0; // far plane distance

    for (Sphere sp:sphereVector) intersect( r, sp , n , maxDist);

    return maxDist < 10000.0;
}


}; // end of namespace VgMath

